# 
# xenopy.rpc - Generic jabber-rpc code
# Copyright (C) 2005 Jason Chu <jason@archlinux.org>
# 
#   This program is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
# 

import SimpleXMLRPCServer
import jabber
import xmpp
import xmpp.simplexml
import xmpp.protocol
import xmlrpclib
import exceptions
import time

class NoReturn(exceptions.Exception):
	pass

class NoResponse(exceptions.Exception):
	pass

class JabberXMLRPCHandler(SimpleXMLRPCServer.SimpleXMLRPCDispatcher):
	def __init__(self):
		SimpleXMLRPCServer.SimpleXMLRPCDispatcher.__init__(self)

	def handle_xmlrpc(self, request_text):
		response = self._marshaled_dispatch(request_text)
		return response

handler = JabberXMLRPCHandler()
rxonly = 0

def registerRPC(onlyrx=0):
	global rxonly
	jabber.conn.RegisterHandler('iq', iqCB)
	rxonly = onlyrx

def iqCB(conn,iqNode):
	if iqNode.getQueryNS() == xmpp.NS_RPC:
		typ = iqNode.getType()
		if typ == "set" and rxonly == 0:
			for i in iqNode.getQueryPayload():
				if isinstance(i,xmpp.simplexml.Node):
					handler.frm = iqNode.getFrom().__str__()
					payload = "".join(handler.handle_xmlrpc(str(i)).split("\n")[1:])
					reply = iqNode.buildReply('result')
					newpay = xmpp.simplexml.XML2Node(payload)
					reply.setQueryPayload([newpay])
					conn.send(str(reply))
					raise xmpp.NodeProcessed

def responseHandler(conn,iqNode):
	if iqNode.getQueryNS() == xmpp.NS_RPC:
		typ = iqNode.getType()
		if typ == "result":
			p, u = xmlrpclib.getparser()
			[p.feed(i.__str__()) for i in iqNode.getQueryPayload()]
			p.close()
			return u.close()
	return None

def register_function(fun, name=None):
	handler.register_function(fun, name)

def register_instance(inst):
	handler.register_instance(inst)

class ServerProxy:
	def __init__(self, jid):
		self.jid = xmpp.JID(jid)

	def __request(self, methodname, params):
		request = "".join(xmlrpclib.dumps(params, methodname).split("\n")[1:])

		resp = jabber.sendMessage(xmpp.protocol.Iq(typ='set', queryNS=xmpp.NS_RPC, to=self.jid, payload=[xmpp.simplexml.XML2Node(request)]), wait=10)
		if resp == None:
			raise NoResponse()
		else:
			return responseHandler(jabber.conn,resp)

	def __getattr__(self, name):
		return xmlrpclib._Method(self.__request, name)
